#!/bin/sh
# The DFA matcher would wrongly convert a regular expression like
# a+a+a to a+a, thus possibly reporting a false match.
# Introduced in v0.1-2111-g4299106ce

# Copyright 2020-2026 Free Software Foundation, Inc.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

. "${srcdir=.}/init.sh"; path_prepend_ ../src

# Add "." to PATH for the use of test-dfa-match-aux.
path_prepend_ .

fail=0

LC_ALL=C ${CHECKER} test-dfa-match-aux 'x+x+x+' xx > out 2>&1 || fail=1
compare /dev/null out || fail=1

Exit $fail
